use Getopt::Long;
use Test::More;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Error qw( GetError ExistsError Error );
use DE_EPAGES::Database::API::Connection qw( RunOnStore GetTestSite  );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw( ImportObjects );
use DE_EPAGES::Object::API::Factory qw(LoadObjectByPath ExistsObjectByPath LoadRootObject);
use DE_EPAGES::Core::API::Object::DateTimeFormatter;
use DE_EPAGES::Chronopost::API::RemoteChronopostTransaction;
use DE_EPAGES::Chronopost::API::SOAP::RemoteChronopost;

 use Data::Dumper;

# use DE_EPAGES::Chronopost::t::MockupResponses;
use constant CORRECT_RESPONSE => 200;

use strict;

my $ROOTPATH = '/Test';


sub run_tests {

    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            # initializsation
            ok( Init(), 'initialization' );

            eval {
                my ($continue, $hObjects) = testImport();
                return unless ( $continue );

                $hObjects->{'remoteTransaction'} =
                    DE_EPAGES::Chronopost::API::RemoteChronopostTransaction->new($hObjects->{'shipFM'},
                    $hObjects->{'transaction'});

                my $aShippingLabels = getShippingLabels($hObjects);
                return unless ( $aShippingLabels );

                testShippingService($hObjects, $aShippingLabels);

                my $zipcode = '75000';
                testPointRelaisService($hObjects, $zipcode);

            };
            if (ExistsError()) {
                my $Error = GetError();
                fail('Error - '. $Error->code);
                diag($Error->longMessage);
                # always cleanup
                ok( Cleanup(), 'cleanup');
                Error($Error);
            }
            # cleanup
            ok( Cleanup(), 'cleanup');
        }
    );
}

sub testImport{
    my $testImportOK = 0;

    my $Shop = LoadObjectByPath($ROOTPATH)->child('TestShopChronopost');
    $testImportOK = isa_ok($Shop, 'DE_EPAGES::Shop::API::Object::Shop');

    my $ShippingFulfillmentMethod = $Shop->path('ShippingFulfillmentMethods/ChronopostLogistics');
    $testImportOK = isa_ok($ShippingFulfillmentMethod,
    'DE_EPAGES::Chronopost::API::Object::ShippingFulfillmentMethodChronopost') && $testImportOK;

    my $Customer = $Shop->path('Customers/TestCustomer');
    $testImportOK = isa_ok($Customer, 'DE_EPAGES::Customer::API::Object::Customer' ) && $testImportOK;

    my $Order = $Shop->path('Customers/TestCustomer/Orders/1234-5678');
    $testImportOK = isa_ok($Order, 'DE_EPAGES::Order::API::Object::CustomerOrder' ) && $testImportOK;

    $testImportOK = cmp_ok(scalar @{$Order->get('Documents')}, '>=',1, 'number of orderdocuments') && $testImportOK;

    my $ShippingLabel = $Shop->path('Customers/TestCustomer/Orders/1234-5678/ShippingLabels/1234-5678-1');
    $testImportOK = isa_ok($ShippingLabel, 'DE_EPAGES::Chronopost::API::Object::ShippingLabelChronopost' )
    && $testImportOK;

    my $Transaction = $Shop->path('ShippingOrders/ShippingOrderChronopost');
    $testImportOK = isa_ok($Transaction, 'DE_EPAGES::ExternalShipping::API::Object::ShippingTransaction' )
                        && $testImportOK;

    $testImportOK = cmp_ok(scalar @{$Transaction->get('ShippingLabels')}, '>=',1, 'number of labels') && $testImportOK;

    foreach my $ShippingLabel (@{$Transaction->get('ShippingLabels')}) {
        $testImportOK = isa_ok($ShippingLabel, 'DE_EPAGES::Chronopost::API::Object::ShippingLabelChronopost' )
        && $testImportOK;
    }
    return ($testImportOK, {    'shop'          => $Shop,
                                'shipFM'        => $ShippingFulfillmentMethod,
                                'customer'      => $Customer,
                                'order'         => $Order,
                                'transaction'   => $Transaction,
    });
}

sub getShippingLabels{
    my ($hObjects) = @_;

    my $Transaction = $hObjects->{'transaction'};

    my $ShippingLabels = $Transaction->get('ShippingLabels');

    return $ShippingLabels;
}

sub testShippingService{
    my ($hObjects, $aShippingLabels) = @_;

    my $RemoteTransaction = $hObjects->{'remoteTransaction'};

    foreach my $hShipment (@$aShippingLabels) {

        my $hResult = $RemoteTransaction->wssetShipping($hShipment);

        my $decFile = $hResult->{'decFile'};

        cmp_ok($hResult->{'response'},'==', CORRECT_RESPONSE,
        'Response on shipping label ');
        cmp_ok($hResult->{'errorCode'},'==', 0, 'Shipping on shipping label ');
        cmp_ok($decFile,'>=', 0, 'Exists label on shipping label ');
    }
}

sub testPointRelaisService{
    my ($hObjects, $zipcode) = @_;

    my $RemoteTransaction = $hObjects->{'remoteTransaction'};
    my $username = $hObjects->{'shipFM'}->get('WebserviceUsername');
    my $password = $hObjects->{'shipFM'}->get('WebservicePassword');

    my $hResult = $RemoteTransaction->wsgetPointRelais($username, $password, $zipcode);
    my $pointRelaisList = $hResult->{'pointRelaisList'};

    cmp_ok($hResult->{'response'}, '==', CORRECT_RESPONSE, 'Chronopost webservices are up and running');
    cmp_ok($hResult->{'errorCode'}, '==', 0, 'PointRelais service OK');
    cmp_ok($pointRelaisList, '>=', 0, 'Some PointRelais points found');

}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    my $System = LoadRootObject();
    $System->insertChild( 'Object', { Alias => 'Test' } );
    ImportObjects('import.xml', $ROOTPATH);
    return 1;
}

sub usage {
    print <<END_USAGE;
Usage:
    perl $0 flags

flags:
    -help       show the command line options

Examples:
    perl $0

END_USAGE
    exit 2;
}


sub Main {
    my ( $Help );

    GetOptions ('help' => \$Help);

    usage() if $Help;

    run_tests();
    done_testing;
}

RunScript( 'Sub' => \&Main );

1;